%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%




function POVray_export(xx1,xx2,ff,filename)

% xx1 -> meshgrix matrix for positions on axis 1
% xx2 -> meshgrix matrix for positions on axis 2
% ff  -> Field values on XX1, XX2 positions.
% OPTIONS -> used to set light, camera and switch axis on or off.
%            still to be implemented!

 
% 
% ========= WARNING ==============
fprintf('\n');
fprintf('######################################################\n')
fprintf('####                                              ####\n')
fprintf('####         P O V r a y _ e x p o r t ( )        ####\n')
fprintf('####                                              ####\n')
fprintf('#### TO DO: Controllare che un piano piatto (tipo ####\n')
fprintf('####        soluzione nulla) sia esportato bene!! ####\n')
fprintf('######################################################\n')
fprintf('**POVray_export() works with square matrices.\n');
fprintf('**Non-square grids will be cut along the minimum axis.\n')
fprintf('### Axis are arbitrarily plotted in the domain CG,\n')
fprintf('### not in the real Origin (0,0,0)!\n\n')

% Dimensional check
[mm1,nn1] = size(xx1);
[mm2,nn2] = size(xx2);
[mm3,nn3] = size(ff);
cmpBool_mm = (mm1 == mm2) && (mm1 == mm3) && (mm2 == mm3);
cmpBool_nn = (nn1 == nn2) && (nn1 == nn3) && (nn2 == nn3);
cmpBool    = cmpBool_mm && cmpBool_nn;

ErrStr1 = 'Attention!\n';
ErrStr2 = 'Input string dimensions are not consistent!\n';
ErrStr3 = 'Please, consider a passionate confession to the ';
ErrStr4 = 'closest priest.\n';
ErrStr5 = 'In: POVray_export()\n\n';
if cmpBool ~= 1
   error([ErrStr1,ErrStr2,ErrStr3,ErrStr4,ErrStr5]);
end


% Centering into Origin
maxmaxX1 = max(max(xx1));
minminX1 = min(min(xx1));
maxmaxX2 = max(max(xx2));
minminX2 = min(min(xx2));
maxmaxFF = max(max(ff));
minminFF = min(min(ff));


meanX1 = 0.5*( maxmaxX1 + minminX1 );
meanX2 = 0.5*( maxmaxX2 + minminX2 );
meanFF = 0.5*( maxmaxFF + minminFF );

xx1 = xx1 - meanX1;
xx2 = xx2 - meanX2;
ff  = ff  - meanFF;


% Scaling axis:
%   the biggest axis is scaled to 1
%   the smallest axis is scaled accordingly to keep geometrical
%   ratios
%   the field is scaled to 1

X1len = maxmaxX1 - minminX1;
X2len = maxmaxX2 - minminX2;

Xscale = 1/X1len * (X1len >= X2len) + 1/X2len * (X2len > X1len);

xx1 = xx1*Xscale;
xx2 = xx2*Xscale;

fflen = maxmaxFF - minminFF;


% ff going from 0 to 1
% and remember: POVray follows left hand rule!!!!

if fflen > 1e-10 % The solution may be flat! Do not divide 
                 % for zero, then!
   FFscale = 1/fflen;
   ff      = -ff*FFscale - 0.5;
else
   FFscale = 0; % If solution is flat, this will make it zero
   ff = -ff*FFscale;
end



% Parameters..
GridSize = min(size(xx1,1),size(xx1,2));

% Cutting matrices..
xx1 = xx1(1:GridSize,1:GridSize);
xx2 = xx2(1:GridSize,1:GridSize);
ff  = ff(1:GridSize, 1:GridSize);


% Opening file in writing mode
fid = fopen(filename,'w');


% Camera and Light Source
fprintf(fid,'camera{\n');
fprintf(fid,'       location <1,1,-2>\n');
fprintf(fid,'       sky <0,0,-1>');
fprintf(fid,'       look_at <0,0,-0.5>}\n');
fprintf(fid,'light_source {<-10,20,-50>, rgb 1}\n');
fprintf(fid,'\n');

% Declarations 
fprintf(fid,'#declare GridSize = %d;\n', GridSize);
fprintf(fid,'#declare PointCount = pow(GridSize,2);\n');
fprintf(fid,'#declare NumberOfFaces = 2*pow(GridSize-1,2);\n');

fprintf(fid,'\n');

% X points array
fprintf(fid,'#declare XCoord = array[PointCount] {\n');
fprintf(fid,' %f,\n',xx1(1:end-1));
fprintf(fid,' %f}\n',xx1(end));

fprintf(fid,'\n#declare YCoord = array[PointCount] {\n');
fprintf(fid,' %f,\n',xx2(1:end-1));
fprintf(fid,' %f}\n',xx2(end));

fprintf(fid,'\n#declare ZCoord = array[PointCount] {\n');
fprintf(fid,' %f,\n',ff(1:end-1));
fprintf(fid,' %f}\n',ff(end));

fprintf(fid,'\n');

% Now printing some more stuff..
fprintf(fid,'mesh2 {\n  vertex_vectors {\n');
fprintf(fid,'    PointCount,\n');
fprintf(fid,'    #local I = 0;\n    #while (I<PointCount)\n');
fprintf(fid,'      #if (I!=PointCount-1)\n');
fprintf(fid,'        <XCoord[I],YCoord[I],ZCoord[I]>,\n');
fprintf(fid,'      #else\n');
fprintf(fid,'        <XCoord[I],YCoord[I],ZCoord[I]>\n');
fprintf(fid,'      #end\n');
fprintf(fid,'      #local I = I + 1;\n');
fprintf(fid,'    #end\n');
fprintf(fid,'  }\n');
fprintf(fid,'  face_indices {\n');
fprintf(fid,'    NumberOfFaces\n');
fprintf(fid,'    #local I = 0;\n');
fprintf(fid,'    #while (I<GridSize-1)\n');
fprintf(fid,'      #local J = 0;\n');
fprintf(fid,'      #while (J<GridSize-1)\n');
fprintf(fid,'        #local K = I*(GridSize)+J;\n');
fprintf(fid,'        <K,K+1,K+GridSize>,\n');
fprintf(fid,'        #if (K!=NumberOfFaces-1)\n');
fprintf(fid,'          <K+1,K+GridSize+1,K+GridSize>,\n');
fprintf(fid,'        #else\n');
fprintf(fid,'          <K+1,K+GridSize+1,K+GridSize>\n');
fprintf(fid,'        #end\n');
fprintf(fid,'        #local J = J + 1;\n');
fprintf(fid,'      #end\n');
fprintf(fid,'      #local I = I + 1;\n');
fprintf(fid,'    #end\n');
fprintf(fid,'  }\n');
%fprintf(fid,'  pigment {rgb 1}\n');
fprintf(fid,'  pigment {\n');
fprintf(fid,'    gradient <0,0,-1>\n');
fprintf(fid,'    color_map {[0 color rgb<0,0,1>]\n');
fprintf(fid,'               [0.5 color rgb<0,1,0>]\n');
fprintf(fid,'               [1 color rgb<1,0,0>]}\n');
fprintf(fid,'  } // end of pigment\n');
fprintf(fid,'}\n');

fprintf(fid,'\n');


% Adding "Axis"
fprintf(fid,'// X arrow\n');
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <-0.8,0,0>,<0.8,0,0>,0.003 }\n');
fprintf(fid,'         cone { <0.8,0,0>,0.02,<0.85,0,0>,0 }\n');
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');

fprintf(fid,'// Y arrow\n');
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <0,-0.8,0>,<0,0.8,0>,0.003 }\n');
fprintf(fid,'         cone { <0,0.8,0>,0.02,<0,0.85,0>,0 }\n');
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');

fprintf(fid,'// Z arrow\n'); %Remember, POVray uses left hand rule!
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <0,0,0.8>,<0,0,-1.2>,0.003 }\n');
fprintf(fid,'         cone { <0,0,-1.2>,0.02,<0,0,-1.25>,0 }\n');
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');



fclose(fid);


return
