%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%



% This function plots the geometry!


xcam = 1.5;
ycam = 1.5;
zcam = -0.8;

% Object colors
%Rcol = 0;
%Gcol = 1;
%Bcol = 1;

Rcol = 0.753;
Gcol = 0.753;
Bcol = 0.753;

%Rcol = 0.82;
%Gcol = 0.55;
%Bcol = 0.2;

shiny = 1;

filename = 'POVgeo.pov';




% Step 1: Domain dimension (we want to see something!)



Xmin = min(min(totalSups.rq(:,1)));
Xmax = max(max(totalSups.rq(:,1)));
Ymin = min(min(totalSups.rq(:,2)));
Ymax = max(max(totalSups.rq(:,2)));
Zmin = min(min(totalSups.rq(:,3)));
Zmax = max(max(totalSups.rq(:,3)));

Xlen = Xmax - Xmin;
Ylen = Ymax - Ymin;
Zlen = Zmax - Zmin;

Xmean = 0.5*(Xmin + Xmax);
Ymean = 0.5*(Ymin + Ymax);
Zmean = 0.5*(Zmin + Zmax);



% Camera Position
Xcam = Xmean + 1.5*Xlen/2*xcam;
Ycam = Ymean + 1.5*Ylen/2*ycam;
Zcam = Zmean + 1.5*Zlen/2*zcam;


% Light Source Position
Xl = 20*Xlen + Xmean;
Yl = 20*Ylen + Ymean;
Zl = -50*Zlen + Zmean;


% Initializing file
fid = fopen(filename, 'w');


% Camera and Light Source
fprintf(fid,'camera{\n');
fprintf(fid,'       location <%f,%f,%f>\n',Xcam,Ycam,Zcam);
fprintf(fid,'       sky <0,0,-1>'); % Roll-aligned with horizon
fprintf(fid,'       look_at <%f,%f,%f>}\n',Xmean,Ymean,Xmean);
fprintf(fid,'light_source {<%f,%f,%f>, rgb 1}\n',Xl,Yl,Zl);
fprintf(fid,'\n');


% Adding "Axis"
Rax = 1/800*sqrt(Xlen^2 + Ylen^2 + Zlen^2);

fprintf(fid,'// X arrow\n');
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <%f,0,0>,<%f,0,0>,%f }\n',Xmin/2,Xmax/2,Rax);
fprintf(fid,'         cone { <%f,0,0>,%f,<%f+%f,0,0>,0 }\n',Xmax/2,Rax*4,Xmax/2,Rax*8);
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');

fprintf(fid,'// Y arrow\n');
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <0,%f,0>,<0,%f,0>,%f }\n',Ymin/2,Ymax/2,Rax);
fprintf(fid,'         cone { <0,%f,0>,%f,<0,%f+%f,0>,0 }\n',Ymax/2,Rax*4,Ymax/2,Rax*8);
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');

fprintf(fid,'// Z arrow\n'); %Remember, POVray uses left hand rule!
fprintf(fid,'union {\n');
fprintf(fid,'         cylinder { <0,0,%f>,<0,0,%f>,%f }\n',-Zmin/2,-Zmax/2,Rax);
fprintf(fid,'         cone { <0,0,%f>,%f,<0,0,%f-%f>,0 }\n',-Zmax/2,Rax*4,-Zmax/2,Rax*8);
fprintf(fid,'         pigment { rgb<1,0,0> }\n');
fprintf(fid,'}\n');
fprintf(fid,'\n');



% Now Adding Objects

for kk = 1:numel(sups)

   fprintf(fid,'\n');

   typeNow = sups(kk).type;
   
   
   
   
   % ========= SPHERE ============
   if strcmp(typeNow,'sphere')
   
      Rad = sups(kk).Rad;
      xyz = sups(kk).xyz;
      
      x = xyz(1);
      y = xyz(2);
      z = xyz(3);
      
      fprintf(fid,'sphere{<%f,%f,%f>,%f\n',x,y,z,Rad);
      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
      if shiny == 1
         fprintf(fid,'finish {\n');
   %      fprintf(fid,'        diffuse .1\n');
   %      fprintf(fid,'        ambient .5\n');
         fprintf(fid,'        roughness .1\n');
         fprintf(fid,'        metallic\n');
         fprintf(fid,'        reflection {\n');
         fprintf(fid,'                    .45\n');
         fprintf(fid,'                    metallic}\n');
         fprintf(fid,'       }\n');
      end
      fprintf(fid,'}\n');

   
   
   % ========== CONE ============
   elseif strcmp(typeNow,'cone')
   
      vers = sups(kk).vers;
      hh   = sups(kk).Len;
      xyz  = sups(kk).xyz;
      
      c1 = xyz(1) - hh/2*vers(1)/norm(vers); % Xbase
      c2 = xyz(2) - hh/2*vers(2)/norm(vers); % Ybase
      c3 = xyz(3) - hh/2*vers(3)/norm(vers); % Zbase
      
      c4 = sups(kk).Rbase;

      c5 = xyz(1) + hh/2*vers(1)/norm(vers); % Xbase
      c6 = xyz(2) + hh/2*vers(2)/norm(vers); % Ybase
      c7 = xyz(3) + hh/2*vers(3)/norm(vers); % Zbase
      
      c8 = sups(kk).Rtop;
      
      % Remember: Left hand rule! Z axis inverted!!!
      %           -c3 and -c7!!
      fprintf(fid,'cone{<%f,%f,%f>,%f,<%f,%f,%f>,%f \n',c1,c2,-c3,c4,c5,c6,-c7,c8);
      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
      if shiny == 1
         fprintf(fid,'finish {\n');
   %      fprintf(fid,'        diffuse .1\n');
   %      fprintf(fid,'        ambient .5\n');
         fprintf(fid,'        roughness .1\n');
         fprintf(fid,'        metallic\n');
         fprintf(fid,'        reflection {\n');
         fprintf(fid,'                    .45\n');
         fprintf(fid,'                    metallic}\n');
         fprintf(fid,'       }\n');
      end
      fprintf(fid,'}\n');
      
      
      
      
      
   % ============ PLANE ==============
   elseif strcmp(typeNow,'plane')
   
      orient = sups(kk).orient;

      if (strcmp(orient,'xy')) || (strcmp(orient,'yx'))
         xPmin = sups(kk).x1min;
	 xPmax = sups(kk).x1max;
	 yPmin = sups(kk).x2min;
	 yPmax = sups(kk).x2max;
	 zPmin = sups(kk).x3val;
	 zPmax = sups(kk).x3val;
	 
	 P1 = [xPmin,yPmin,zPmin];
	 P2 = [xPmax,yPmin,zPmin];
	 P3 = [xPmax,yPmax,zPmin];
	 P4 = [xPmin,yPmax,zPmin];
      elseif (strcmp(orient,'yz')) || (strcmp(orient,'zy'))
         xPmin = sups(kk).x3val;
	 xPmax = sups(kk).x3val;
	 yPmin = sups(kk).x1min;
	 yPmax = sups(kk).x1max;
	 zPmin = sups(kk).x2min;
	 zPmax = sups(kk).x2max;
	 
	 P1 = [xPmin,yPmin,zPmin];
	 P2 = [xPmin,yPmax,zPmin];
	 P3 = [xPmin,yPmax,zPmax];
	 P4 = [xPmin,yPmin,zPmax];
      elseif (strcmp(orient,'xz')) || (strcmp(orient,'zx'))
         xPmin = sups(kk).x1min;
	 xPmax = sups(kk).x1max;
	 yPmin = sups(kk).x3val;
	 yPmax = sups(kk).x3val;
	 zPmin = sups(kk).x2min;
	 zPmax = sups(kk).x2max;
	 
	 P1 = [xPmin,yPmin,zPmin];
	 P2 = [xPmax,yPmin,zPmin];
	 P3 = [xPmax,yPmin,zPmax];
	 P4 = [xPmin,yPmin,zPmax];
      end

      fprintf(fid,'polygon{\n');
      fprintf(fid,'        5,<%f,%f,%f> ',P1(1),P1(2),P1(3));
      fprintf(fid,'<%f,%f,%f> ',P2(1),P2(2),P2(3));
      fprintf(fid,'<%f,%f,%f> ',P3(1),P3(2),P3(3));
      fprintf(fid,'<%f,%f,%f> ',P4(1),P4(2),P4(3));
      fprintf(fid,'<%f,%f,%f>\n',P1(1),P1(2),P1(3));
      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
      if shiny == 1
         fprintf(fid,'finish {\n');
   %      fprintf(fid,'        diffuse .1\n');
   %      fprintf(fid,'        ambient .5\n');
         fprintf(fid,'        roughness .1\n');
         fprintf(fid,'        metallic\n');
         fprintf(fid,'        reflection {\n');
         fprintf(fid,'                    .45\n');
         fprintf(fid,'                    metallic}\n');
         fprintf(fid,'       }\n');
      end
      fprintf(fid,'}\n');


   
   % ============ CYLINDER ============
   elseif strcmp(typeNow,'cylinder')
   
      vers = sups(kk).vers;
      hh   = sups(kk).Len;
      xyz  = sups(kk).xyz;
      
      c1 = xyz(1) - hh/2*vers(1)/norm(vers); % Xbase
      c2 = xyz(2) - hh/2*vers(2)/norm(vers); % Ybase
      c3 = xyz(3) - hh/2*vers(3)/norm(vers); % Zbase
      
      c4 = xyz(1) + hh/2*vers(1)/norm(vers); % Xbase
      c5 = xyz(2) + hh/2*vers(2)/norm(vers); % Ybase
      c6 = xyz(3) + hh/2*vers(3)/norm(vers); % Zbase
      
      c7 = sups(kk).Rad;
      
      % Remember: Left hand rule! Z axis inverted!!!
      %           -c3 and -c7!!
      fprintf(fid,'cylinder{<%f,%f,%f>,<%f,%f,%f>,%f \n',c1,c2,-c3,c4,c5,-c6,c7);
      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
      fprintf(fid,'}\n');
   
   
   
   
   % ============= LINE ==============
   elseif strcmp(typeNow,'line') % Small cylinder..

      xyzA = sups(kk).xyzA;
      xyzB = sups(kk).xyzB;
      
      xA = xyzA(1);
      yA = xyzA(2);
      zA = xyzA(3);
      
      xB = xyzB(1);
      yB = xyzB(2);
      zB = xyzB(3); 

      Rad = 1/500*sqrt(Xlen^2 + Ylen^2 + Zlen^2);

      fprintf(fid,'cylinder{<%f,%f,%f>,<%f,%f,%f>,%f \n',xA,yA,-zA,xB,yB,-zB,Rad);
      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
      if shiny == 1
         fprintf(fid,'finish {\n');
   %      fprintf(fid,'        diffuse .1\n');
   %      fprintf(fid,'        ambient .5\n');
         fprintf(fid,'        roughness .1\n');
         fprintf(fid,'        metallic\n');
         fprintf(fid,'        reflection {\n');
         fprintf(fid,'                    .45\n');
         fprintf(fid,'                    metallic}\n');
         fprintf(fid,'       }\n');
      end
      fprintf(fid,'}\n');


   
   % ============ LOOP ============
   elseif strcmp(typeNow,'loop') % Torus!!

      fprintf('Attention: wire loop geometry exportation in .pov');
      fprintf(' format is not supported yet!');
      fprintf('Omitting wire loop. Sorry about that.\n');

%      Rmin = 1/500*sqrt(Xlen^2 + Ylen^2 + Zlen^2);
%      Rmax = sups(kk).Rad;
%      vv   = sups(kk).vers;
%      vv   = vv/norm(vv);
%      xyz  = sups(kk).xyz;
%      
%      
%      rotVers = cross([0,0,1],vv);
%      PSI1 = rotVers(1);
%      PSI2 = rotVers(2);
%      PSI3 = rotVers(3);
%            
%      % Euler-Rodriguez formula
%      kcross = [0, -PSI3, PSI2; PSI3, 0, -PSI1; -PSI2, PSI1, 0];
%      thetaa = asin(norm(rotVers));
%
%      RRR = eye(3) + sin(thetaa)*kcross + (1-cos(thetaa))*kcross*kcross;
%
%      if (abs(RRR(3,1)) ~= 1) 
%         theta1 = -asin(RRR(3,1));
%%	 theta2 = pi - theta1;
%	 psi1   = atan2(RRR(3,2)/cos(theta1),RRR(3,3)/cos(theta1));
%%	 psi2   = atan2(RRR(3,2)/cos(theta2),RRR(3,3)/cos(theta2));
%	 phi1   = atan2(RRR(2,1)/cos(theta1),RRR(1,1)/cos(theta1));
%%	 phi2   = atan2(RRR(2,1)/cos(theta2),RRR(1,1)/cos(theta1));
%
%         psi   = psi1;
%	 theta = theta1;
%	 phi   = phi1;
%      else
%         phi = 0;
%	 if RRR(3,1) == -1
%	    theta = pi/2;
%	    psi   = phi + atan2(RRR(1,2),RRR(1,3));
%	 else
%	    theta = -pi/2;
%	    psi   = -phi + atan2(-RRR(1,2),-RRR(1,3));
%	 end
%      end
%
%      psi   = 180/pi*psi;
%      theta = 180/pi*theta;
%      phi   = 180/pi*phi;
%      
%      
%      fprintf(fid,'torus{');
%      fprintf(fid,'%f, %f\n',Rmax,Rmin);
%      fprintf(fid,'rotate <%f,%f,%f>\n',psi,theta,phi);
%%      fprintf(fid,'translate <%f,%f,%f>\n',xyz(1),xyz(2),xyz(3));
%      fprintf(fid,'translate <%f,%f,%f>\n',0,0,0);
%      fprintf(fid,'pigment{rgb<%f,%f,%f>}\n',Rcol,Gcol,Bcol);
%      fprintf(fid,'}\n');
%   

   end




end




% Closing file
fclose(fid);

