%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% capacity calculator


fprintf('\n====== Capacity Calculator ======\n');

% These are the implemented steps, to be done
% one object after the other.
% 
% Let kk be the current object
%
% STEP 0: set the voltage of EVERY object to floating 
%         value 'free'
%         set the total charge Q of every object EXCEPT kk
%         to 0
%         set the total charge of the kk-th object to 1
%
% STEP 1: solve the linear system and find the electric
%         potentials induced by the charge on kk:
%         these determine a column for elastance matrix
%
% STEP 2: computing elastance coefficients.
%         Since we are working with charges that are defined
%         as Q = q/eps0, the elastance value is
%         S_ij = v_i / q_j = v_i / (Q_j eps0)
%        
% STEP 3: computing capacitance coefficients matrix that is
%         simply inv(ss).
%         Since ss is small, it's not hard inverting it.
%
% STEP 4: computing the capacities
%


% Initializing elastance and capacitance matrices
ss = zeros(numel(sups),numel(sups));
cc = zeros(numel(sups),numel(sups));

% Now cloning our little friend totalSups
totalSups_TRUE = totalSups; % Backup

totalSups.nSupsImposed  = 0;
totalSups.nSupsFloating = numel(sups);

% and backupping simulation original results..
Vresult_vect = {};
for kk_C = 1:numel(sups)
   Vresult_vect{kk_C} = sups(kk_C).Vresult;
end


% Ok.. for every body:
for kk_C = 1:numel(sups)

   fprintf('Solving with charge imposed on object %d... ',kk_C);

   % ----- STEP 0 -----
   
   % Setting the total charge on every surface to 0...
   for jj_C = 1:numel(sups)
      sups(jj_C).Q = 0;
      sups(jj_C).V = 'free';
   end
   
   % ... except the kk-th surface
   sups(kk_C).Q = 1;
   
   
   % ----- STEP 1 -----
   
   % Setting verbosity to 0, or the solver script will be 
   % noisy
   
   verbosity = 0;
   
   sol_calculator;
   
   fprintf('Done!\n');
   
   % ----- STEP 2 -----
   
   % Filling the elastance matrix
   
   for jj_C = 1:numel(sups)
      ss(jj_C,kk_C) = sups(jj_C).Vresult / 1;
   end
   

end

fprintf('\n')
fprintf('Elastance matrix computed: Sij = Vi/(Qj*Epsilon0)\n')

% ----- STEP 3 -----
% Computing cc, capacitance coefficients matrix

cc = inv(ss);
fprintf('Capacitance matrix computed: C = inv(S) [Coulomb*Epsilon0 / Volts]\n')


% ----- STEP 4 -----
% Computing Capacities
CapMat = -cc;  % Extra-diagonal elements are the same
CapMat = CapMat - diag(diag(CapMat)); % Diagonal elements are 0 for now
diagV  = diag(cc) - sum(CapMat,2);
CapMat = CapMat + diag(diagV);

fprintf('Capacity matrix computed!! C = (q/Epsilon0)/V\n')
disp(CapMat)




% ------ NOW SOME COURTESY FINAL TREATMENT ------

% When finished, restoring input parameters
totalSups = totalSups_TRUE;

for kk_C = 1:numel(sups)
   sups(kk_C).Vresult = Vresult_vect{kk_C};
end

INPUT
input_filler


%fprintf('Solving the real system one last time, ')
%fprintf('to restore original values.\n')
%
%sol_calculator


% Now saving computed matrices and capacity
totalSups.ss     = ss;
totalSups.cc     = cc;
totalSups.CapMat = CapMat;



