%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%






function [XYZq, XYZc, XXc, YYc, ZZc, Amat, NN] = cone_creator(Rbase,Rtop,L,x,y,z,versor,n_th,n_z)

% ==== Cylinder making ====
% 
%
% Step 1: creating a cylinder
% Step 2: rotating the cylinder
% Step 3: creating and rotating normals
% Step 4: creating Areas


%Rbase=2;Rtop=0.2;L=3;x=0;y=0;z=0;versor=[0,1,1];n_th=20;n_z=10;



Rvect = linspace(Rbase,Rtop,n_z+1)';

thvect0 = linspace(0,2*pi,n_th+1)';
thvect = thvect0(1:end-1);

zvect = linspace(-L/2,L/2,n_z+1)';

deltaTh = 2*pi/n_th;
deltaZ  = L/n_z;
d_ar = 0.5 * sqrt(deltaTh*min(Rbase,Rtop)*deltaZ); % Distance between control
                                   % points and surface
                                 


% reshaping..
rMAT = repmat(Rvect,1,n_th);
zMAT = repmat(zvect,1,n_th);
thMAT  = repmat(thvect',n_z+1,1);                                     
%rthzMAT = [R*ones(1,numel(zMAT)), thMAT(1:end), zMAT(1:end)];


xxq_O = rMAT.*sin(thMAT);
yyq_O = rMAT.*cos(thMAT);
zzq_O = zMAT;

xxc_O = (rMAT+d_ar).*sin(thMAT);
yyc_O = (rMAT+d_ar).*cos(thMAT);
zzc_O = zMAT;



XYZq_O = [xxq_O(1:end)',yyq_O(1:end)',zzq_O(1:end)'];
XYZc_O = [xxc_O(1:end)',yyc_O(1:end)',zzc_O(1:end)'];


% ------- Rotating the cylinder -------
versor = versor/norm(versor); % Now it's a versor.
rotVers = cross([0,0,1],versor);
%PSI1 = versor(1);
%PSI2 = versor(2);
%PSI3 = versor(3);
PSI1 = rotVers(1);
PSI2 = rotVers(2);
PSI3 = rotVers(3);


% Creating quaternion and rotating
%Quat = angle2quat(PSI3, PSI2, PSI1);
%RotatedXYZq = quatrotate(Quat, XYZq_O);
%RotatedXYZc = quatrotate(Quat, XYZc_O);


% Euler-Rodriguez formula
kcross = [0, -PSI3, PSI2; PSI3, 0, -PSI1; -PSI2, PSI1, 0];
thetaa = asin(norm(rotVers));

RRR = eye(3) + sin(thetaa)*kcross + (1-cos(thetaa))*kcross*kcross;
RotatedXYZq = (RRR*XYZq_O')';
RotatedXYZc = (RRR*XYZc_O')';

xxc_Or = reshape(RotatedXYZq(:,1), size(xxc_O,1), size(xxc_O,2));
yyc_Or = reshape(RotatedXYZq(:,2), size(xxc_O,1), size(xxc_O,2));
zzc_Or = reshape(RotatedXYZq(:,3), size(xxc_O,1), size(xxc_O,2));


% Closing the cylinder for plotting vectors
XXc = [xxc_Or, xxc_Or(:,1)];
YYc = [yyc_Or, yyc_Or(:,1)];
ZZc = [zzc_Or, zzc_Or(:,1)];

% Traslating everything
XYZq = RotatedXYZq + repmat([x,y,z],size(RotatedXYZq,1),1);
XYZc = RotatedXYZc + repmat([x,y,z],size(RotatedXYZq,1),1);

XXc = XXc + x;
YYc = YYc + y;
ZZc = ZZc + z;




% ---- Computing Normals ----

phi = atan2(Rbase-Rtop,L);

Nxx = sin(thMAT);
Nyy = cos(thMAT);
Nzz = sin(phi)*ones(size(Nxx));

NN = [Nxx(1:end)',Nyy(1:end)',Nzz(1:end)'];
%NN = quatrotate(Quat,NN);
NN = (RRR*NN')';



% ---- Computing area ----

%Aele = R*acos(deltaTh) * deltaZ;
crf = 2*pi*Rvect;
Aele = crf/n_th * deltaZ;
%Amat = Aele * ones(size(XXc));
Amat = repmat(Aele,1,n_th+1);



%figure, plot3(XYZq(:,1), XYZq(:,2), XYZq(:,3),'og'), grid on, hold on
%plot3(XYZc(:,1),XYZc(:,2),XYZc(:,3),'*k')
%
%figure, surf(XXc,YYc,ZZc), axis equal, hold on, grid on
%quiver3(XYZc(:,1),XYZc(:,2),XYZc(:,3),NN(:,1),NN(:,2),NN(:,3));

return
