%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% field_plotter

fprintf('Plotting computed Electric Field on planes.. ');

% Creating plane vectors
xvField = linspace(plotField.xorig, plotField.xorig + plotField.Lx, plotField.nx);
yvField = linspace(plotField.yorig, plotField.yorig + plotField.Ly, plotField.ny);
zvField = linspace(plotField.zorig, plotField.zorig + plotField.Lz, plotField.nz);

[YY1, ZZ1] = meshgrid(yvField, zvField);
XX1 = plotField.xorig * ones(size(YY1));

[XX2, ZZ2] = meshgrid(xvField, zvField);
YY2 = plotField.yorig * ones(size(XX2));

[XX3, YY3] = meshgrid(xvField, yvField);
ZZ3 = plotField.zorig * ones(size(XX3));


% Computing fields on the planes

%VV1 = zeros(size(XX1));
%VV2 = zeros(size(XX2));
%VV3 = zeros(size(XX3));

EEx1 = zeros(size(XX1));
EEy1 = zeros(size(XX1));
EEz1 = zeros(size(XX1));

EEx2 = zeros(size(XX2));
EEy2 = zeros(size(XX2));
EEz2 = zeros(size(XX2));

EEx3 = zeros(size(XX3));
EEy3 = zeros(size(XX3));
EEz3 = zeros(size(XX3));


for jj = 1:totalSups.nele % for every single charge

   qNow = totalSups.q(jj);
   rNow = totalSups.rq(jj,:);

   xxPlane1 = XX1 - rNow(1);
   yyPlane1 = YY1 - rNow(2);
   zzPlane1 = ZZ1 - rNow(3);
   
   xxPlane2 = XX2 - rNow(1);
   yyPlane2 = YY2 - rNow(2);
   zzPlane2 = ZZ2 - rNow(3);

   xxPlane3 = XX3 - rNow(1);
   yyPlane3 = YY3 - rNow(2);
   zzPlane3 = ZZ3 - rNow(3);
   
   
   rPlane1 = sqrt( ( XX1 - rNow(1) ).^2 + ( YY1 - rNow(2) ).^2 + ( ZZ1 - rNow(3) ).^2 );
   rPlane2 = sqrt( ( XX2 - rNow(1) ).^2 + ( YY2 - rNow(2) ).^2 + ( ZZ2 - rNow(3) ).^2 );
   rPlane3 = sqrt( ( XX3 - rNow(1) ).^2 + ( YY3 - rNow(2) ).^2 + ( ZZ3 - rNow(3) ).^2 );

   % singularity check matrix (radius check)
   r1ck = rPlane1 > 0.1;
   r2ck = rPlane2 > 0.1;
   r3ck = rPlane3 > 0.1;
   

   % remember: computed charge is q*epsilon0!!!!!!
   EEx1 = EEx1 + 1/(4*pi) * qNow ./ rPlane1.^3 .* xxPlane1.*r1ck;
   EEy1 = EEy1 + 1/(4*pi) * qNow ./ rPlane1.^3 .* yyPlane1.*r1ck;
   EEz1 = EEz1 + 1/(4*pi) * qNow ./ rPlane1.^3 .* zzPlane1.*r1ck;

   EEx2 = EEx2 + 1/(4*pi) * qNow ./ rPlane2.^3 .* xxPlane2.*r2ck;
   EEy2 = EEy2 + 1/(4*pi) * qNow ./ rPlane2.^3 .* yyPlane2.*r2ck;
   EEz2 = EEz2 + 1/(4*pi) * qNow ./ rPlane2.^3 .* zzPlane2.*r2ck;

   EEx3 = EEx3 + 1/(4*pi) * qNow ./ rPlane3.^3 .* xxPlane3.*r3ck;
   EEy3 = EEy3 + 1/(4*pi) * qNow ./ rPlane3.^3 .* yyPlane3.*r3ck;
   EEz3 = EEz3 + 1/(4*pi) * qNow ./ rPlane3.^3 .* zzPlane3.*r3ck;

   
%   VV1 = VV1 + 1/(4*pi) * qNow ./ rPlane1 .* (rPlane1 > 0.001);
%   VV2 = VV2 + 1/(4*pi) * qNow ./ rPlane2 .* (rPlane2 > 0.001);
%   VV3 = VV3 + 1/(4*pi) * qNow ./ rPlane3 .* (rPlane3 > 0.001);

end


EE1 = sqrt(EEx1.^2 + EEy1.^2 + EEz1.^2);
EE2 = sqrt(EEx2.^2 + EEy2.^2 + EEz2.^2);
EE3 = sqrt(EEx3.^2 + EEy3.^2 + EEz3.^2);



% Saving variables
plotField.XX1 = XX1;
plotField.YY1 = YY1;
plotField.ZZ1 = ZZ1;

plotField.XX2 = XX2;
plotField.YY2 = YY2;
plotField.ZZ2 = ZZ2;

plotField.XX3 = XX3;
plotField.YY3 = YY3;
plotField.ZZ3 = ZZ3;

plotField.VV1 = VV1;
plotField.VV2 = VV2;
plotField.VV3 = VV3;



% PLOTTING!!!!!!!!
% This is quite tricky.. involves permuting matrices and stuff..

[XXX,YYY,ZZZ] = meshgrid(xvField,yvField,zvField);

%VV1p = permute(VV1,[2,3,1]);
%VVV1 = repmat(VV1p,1,numel(xvField),1);
%
%VV2p = permute(VV2,[3,2,1]);
%VVV2 = repmat(VV2p,numel(yvField),1,1);
%
%VV3p = permute(VV3,[1,2,3]);
%VVV3 = repmat(VV3p,1,1,numel(zvField));

EE1p = permute(EE1,[2,3,1]);
EEE1 = repmat(EE1p,1,numel(xvField),1);

EE2p = permute(EE2,[3,2,1]);
EEE2 = repmat(EE2p,numel(yvField),1,1);

EE3p = permute(EE3,[1,2,3]);
EEE3 = repmat(EE3p,1,1,numel(zvField));


figure
cont1 = contourslice(XXX,YYY,ZZZ,EEE1,XX1,YY1,ZZ1,50);
hold on
cont2 = contourslice(XXX,YYY,ZZZ,EEE2,XX2,YY2,ZZ2,50);
cont3 = contourslice(XXX,YYY,ZZZ,EEE3,XX3,YY3,ZZ3,50);

set(cont1, 'linewidth', 2) % Adjusting contourlines width
set(cont2, 'linewidth', 2)
set(cont3, 'linewidth', 2)


axis equal, axis off

MaxV = max([max(max(EE1)), max(max(EE2)), max(max(EE3))]);
MinV = min([min(min(EE1)), min(min(EE2)), min(min(EE3))]);
meanVal = 0.5*(MaxV+MinV);
for kk = 1:numel(sups) % Overimposing objects

   try
   
      surf(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, repmat(MaxV,size(sups(kk).xxc,1),size(sups(kk).xxc,2)));

   catch

      plot3(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, 'k', 'linewidth', 2)
   
   end
      
end

colorbar
xlabel('X','FontSize',14)
ylabel('Y','FontSize',14)
zlabel('Z','FontSize',14)
title('Electric Field Modulus', 'FontSize',15)





figure
subplot(2,2,1)
yzplot = surf(EE1,YY1,ZZ1);
xlabel('Electric field'), ylabel('Y axis'),zlabel('Z axis')
shading interp
set(yzplot,'FaceLighting','phong','FaceColor','interp',...
'AmbientStrength',0.5)
light('Position',[1 0 0],'Style','infinite');

subplot(2,2,2)
xzplot = surf(XX2,EE2,ZZ2);
xlabel('X axis'), ylabel('Electric field'), zlabel('Z axis')
shading interp
set(xzplot,'FaceLighting','phong','FaceColor','interp',...
'AmbientStrength',0.5)
light('Position',[1 0 0],'Style','infinite');

subplot(2,2,3)
xyplot = surf(XX3,YY3,EE3);
xlabel('X axis'), ylabel('Y axis'), zlabel('Electric field')
shading interp
set(xyplot,'FaceLighting','phong','FaceColor','interp',...
'AmbientStrength',0.5)
light('Position',[1 0 0],'Style','infinite');

fprintf('Done!\n')

