%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% Creates geometry by reading the INPUT file


%    MESSAGE
fprintf('\n====== Geometry Creation ======\n');


idnow = 1;

fprintf('Creating input surfaces.. ');
wtCreatingGeo = waitbar(0,'Creating input surfaces..');

for jj = 1:numel(sups) % for every surface..


   waitbar(jj/numel(sups),wtCreatingGeo);


   % =================================
   % reads surface type and creates it
   
   
  
   
   % ------ SPHERE ------
   if strcmp(sups(jj).type, 'sphere')

      Rad  = sups(jj).Rad;
      xyz  = sups(jj).xyz;
      x    = xyz(1);
      y    = xyz(2);
      z    = xyz(3);
      nth  = sups(jj).nth;
      nphi = sups(jj).nphi;
      
      [rq, rc, xxc, yyc, zzc, Amat, nv] = sphere_creator(Rad,x,y,z,nth,nphi); % Creating a sphere
    
    
      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Amat = Amat;
      sups(jj).nv   = nv;  % normals
    
   
   % ------ PLANE ------
   elseif strcmp(sups(jj).type, 'plane')
   
      x1min  = sups(jj).x1min;
      x1max  = sups(jj).x1max;
      x2min  = sups(jj).x2min;
      x2max  = sups(jj).x2max;
      x3val  = sups(jj).x3val;
      n1     = sups(jj).n1;
      n2     = sups(jj).n2;
      orient = sups(jj).orient;
   
      [rq, rc, xxc, yyc, zzc, Amat, nv] = plane_creator(x1min, x1max, x2min, x2max, x3val, n1, n2, orient);
   
      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Amat = Amat;
      sups(jj).nv   = nv;   % normals
   
      
   % ------ CYLINDER ------   
   elseif strcmp(sups(jj).type, 'cylinder')
   
      Rad  = sups(jj).Rad;
      Len  = sups(jj).Len;
      xyz  = sups(jj).xyz;
      x    = xyz(1);
      y    = xyz(2);
      z    = xyz(3);
      nth  = sups(jj).nth;
      nh   = sups(jj).nh;
      vers = sups(jj).vers;
      
      [rq, rc, xxc, yyc, zzc, Amat, nv] = cylinder_creator(Rad,Len,x,y,z,vers,nth,nh);
   
      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Amat = Amat;
      sups(jj).nv   = nv;   % normals
      
      
   % ------ CONE ------   
   elseif strcmp(sups(jj).type, 'cone')
   
      Rbase = sups(jj).Rbase;
      Rtop  = sups(jj).Rtop;
      Len   = sups(jj).Len;
      xyz   = sups(jj).xyz;
      x     = xyz(1);
      y     = xyz(2);
      z     = xyz(3);
      nth   = sups(jj).nth;
      nh    = sups(jj).nh;
      vers  = sups(jj).vers;
      
      [rq, rc, xxc, yyc, zzc, Amat, nv] = cone_creator(Rbase,Rtop,Len,x,y,z,vers,nth,nh);

      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Amat = Amat;
      sups(jj).nv   = nv;   % normals
      
      
   % ------ LINE ------     
   elseif strcmp(sups(jj).type, 'line')
   
      xyzA  = sups(jj).xyzA;
      xyzB  = sups(jj).xyzB;
      Nline = sups(jj).nL;
      xA    = xyzA(1);
      yA    = xyzA(2);
      zA    = xyzA(3);
      xB    = xyzB(1);
      yB    = xyzB(2);
      zB    = xyzB(3);
      
      [rq, rc, xxc, yyc, zzc, Lmat, nv] = line_creator(xA,yA,zA,xB,yB,zB,Nline);
      
      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Lmat = Lmat;
      sups(jj).nv   = nv;
      
      
   % ------ WIRE RING ------     
   elseif strcmp(sups(jj).type, 'loop')
   
      xyz   = sups(jj).xyz;
      Rad   = sups(jj).Rad;
      xyz   = sups(jj).xyz;
      x     = xyz(1);
      y     = xyz(2);
      z     = xyz(3);
      nth   = sups(jj).nth;
      vers  = sups(jj).vers;
      [rq, rc, xxc, yyc, zzc, Lmat, nv] = ring_creator(Rad,x,y,z,vers,nth);
      
      sups(jj).rq   = rq;
      sups(jj).rc   = rc;
      sups(jj).xxc  = xxc;
      sups(jj).yyc  = yyc;
      sups(jj).zzc  = zzc;
      sups(jj).Lmat = Lmat;
      sups(jj).nv   = nv;
      
   end
   
   
   
   % ================================
   % Assigns number of elements
   
   elenumjj = size(sups(jj).rq,1);
   sups(jj).nele    = elenumjj;
   sups(jj).idstart = idnow;
   sups(jj).idend   = idnow + elenumjj - 1;
   
   idnow = idnow + elenumjj;
   
   
   
end

close(wtCreatingGeo)
fprintf('Done!\n');

% ============================
% Creating object TotalGeo

fprintf('Creating total geometry object.. ')

totalSups.rq = [];
totalSups.rc = [];
totalSups.nv = [];

totalSups.neleImposed  = 0; % how many V-imposed nodes
totalSups.neleFloating = 0; % how many V-floating nodes

totalSups.nSupsImposed  = 0; % how many V-imposed surfaces
totalSups.nSupsFloating = 0; % how many V-floating surfaces


for jj = 1:numel(sups)
   totalSups.rq = [totalSups.rq; sups(jj).rq];
   totalSups.rc = [totalSups.rc; sups(jj).rc];
   totalSups.nv = [totalSups.nv; sups(jj).nv];

   if isnumeric(sups(jj).V)
      totalSups.nSupsImposed = totalSups.nSupsImposed + 1;
      totalSups.neleImposed = totalSups.neleImposed+sups(jj).nele;
   elseif strcmp(sups(jj).V, 'free')
      totalSups.nSupsFloating = totalSups.nSupsFloating + 1;
      totalSups.neleFloating =totalSups.neleFloating+sups(jj).nele;
   else
      fprintf('ERROR! One surface was given a wrong value for V\n')
      fprintf('In: geometry_creator\n')
   end
end

totalSups.nele = size(totalSups.rq,1);

fprintf('Done!\n');



