%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





function [XYZq, XYZc, XXc, YYc, ZZc, Lmat, NN] = line_creator(xA,yA,zA,xB,yB,zB,Nline)

% The line goes from [xA,yA,zA] to [xB,yB,zB]

xq = linspace(xA,xB,Nline)';
yq = linspace(yA,yB,Nline)';
zq = linspace(zA,zB,Nline)';

deltaX = xq(2)-xq(1);
deltaY = yq(2)-yq(1);
deltaZ = zq(2)-zq(1);

xc = xq + deltaX/4;
yc = yq + deltaY/4;
zc = zq + deltaZ/4;

deltaL = norm([xB,yB,zB] - [xA,yA,zA]) / Nline;

XYZq = [xq, yq, zq];
XYZc = [xc, yc, zc];

XXc = xc;
YYc = yc;
ZZc = zc;

% Elements length
Lmat = repmat(deltaL, size(xc,1), size(xc,2));

% Normals 
dirVect = [xB-xA, yB-yA, zB-zA];
crossVect = cross(rand(1,3)+rand(1,3), dirVect); % using two rands
                       % makes harder to have one direction that 
		       % is zero!
crossVers = crossVect / norm(crossVect);
NN = repmat(crossVers, size(XXc,1), 1);

return
