%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





function [XYZq, XYZc, XXc, YYc, ZZc, Amat, NN] = plane_creator(x1min, x1max, x2min, x2max, x3val, n1, n2, typee)

% First of all, one chooses the plane orientation:
% typee = 'xy' makes it normal to z.
% typee = 'yz' makes it normal to x.
% typee = 'xz' makes it normal to y.
% The created plane goes from x1min to x1max,
% from x2min to x2max and has a value of 

Deltax1 = x1max - x1min;
Deltax2 = x2max - x2min;

x1 = linspace(x1min,x1max,n1);
x2 = linspace(x2min,x2max,n2);

[XX1,XX2] = meshgrid(x1,x2);
XX3 = x3val*ones(size(XX1));


XX1q = XX1;
XX2q = XX2;
XX3q = XX3;


dx1 = 0.2*( x1(2) - x1(1) );
dx2 = 0.2*( x2(2) - x2(1) );
dx3 = (dx1 + dx2)/2;
XX1c = XX1 + dx1;
XX2c = XX2 + dx2;
XX3c = XX3 + dx3;


if strcmp(typee, 'xy') || strcmp(typee, 'yx')

    XYZq = [XX1q(1:end)', XX2q(1:end)', XX3q(1:end)'];
    XYZc = [XX1c(1:end)', XX2c(1:end)', XX3c(1:end)'];
    XXc  = XX1c;
    YYc  = XX2c;
    ZZc  = XX3c;

    nv1  = 0;
    nv2  = 0;
    nv3  = 1;

elseif strcmp(typee, 'yz') || strcmp(typee,'zy')

    XYZq = [XX3q(1:end)', XX1q(1:end)', XX2q(1:end)'];
    XYZc = [XX3c(1:end)', XX1c(1:end)', XX2c(1:end)'];
    XXc  = XX3c;
    YYc  = XX1c;
    ZZc  = XX2c;

    nv1  = 1;
    nv2  = 0;
    nv3  = 0;


elseif strcmp(typee, 'xz') || strcmp(typee, 'zx')

    XYZq = [XX1q(1:end)', XX3q(1:end)', XX2q(1:end)'];
    XYZc = [XX1c(1:end)', XX3c(1:end)', XX2c(1:end)'];
    XXc  = XX1c;
    YYc  = XX3c;
    ZZc  = XX2c;

    nv1  = 0;
    nv2  = 1;
    nv3  = 0;

end    
   

% Now computing Area (that is uniform over elements!!)
Aele = abs(x1(2)-x1(1)) * abs(x2(2)-x2(1));
Amat = Aele * ones(size(XX1c));


% Normals
NN1 = repmat(nv1,size(XYZq,1),1);
NN2 = repmat(nv2,size(XYZq,1),1);
NN3 = repmat(nv3,size(XYZq,1),1);
NN = [NN1,NN2,NN3];

return
