%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% field_plotter

fprintf('Plotting computed Potential on planes.. ');

% Creating plane vectors
xvField = linspace(plotField.xorig, plotField.xorig + plotField.Lx, plotField.nx);
yvField = linspace(plotField.yorig, plotField.yorig + plotField.Ly, plotField.ny);
zvField = linspace(plotField.zorig, plotField.zorig + plotField.Lz, plotField.nz);

[YY1, ZZ1] = meshgrid(yvField, zvField);
XX1 = plotField.xorig * ones(size(YY1));

[XX2, ZZ2] = meshgrid(xvField, zvField);
YY2 = plotField.yorig * ones(size(XX2));

[XX3, YY3] = meshgrid(xvField, yvField);
ZZ3 = plotField.zorig * ones(size(XX3));


% Computing fields on the planes

VV1 = zeros(size(XX1));
VV2 = zeros(size(XX2));
VV3 = zeros(size(XX3));

for jj = 1:totalSups.nele % for every single charge

   qNow = totalSups.q(jj);
   rNow = totalSups.rq(jj,:);

   rPlane1 = sqrt( ( XX1 - rNow(1) ).^2 + ( YY1 - rNow(2) ).^2 + ( ZZ1 - rNow(3) ).^2 );
   rPlane2 = sqrt( ( XX2 - rNow(1) ).^2 + ( YY2 - rNow(2) ).^2 + ( ZZ2 - rNow(3) ).^2 );
   rPlane3 = sqrt( ( XX3 - rNow(1) ).^2 + ( YY3 - rNow(2) ).^2 + ( ZZ3 - rNow(3) ).^2 );

   % remember: computed charge is q*epsilon0!!!!!!
   VV1 = VV1 + 1/(4*pi) * qNow ./ rPlane1 .* (rPlane1 > 0.001);
   VV2 = VV2 + 1/(4*pi) * qNow ./ rPlane2 .* (rPlane2 > 0.001);
   VV3 = VV3 + 1/(4*pi) * qNow ./ rPlane3 .* (rPlane3 > 0.001);

end



% Saving variables
plotField.XX1 = XX1;
plotField.YY1 = YY1;
plotField.ZZ1 = ZZ1;

plotField.XX2 = XX2;
plotField.YY2 = YY2;
plotField.ZZ2 = ZZ2;

plotField.XX3 = XX3;
plotField.YY3 = YY3;
plotField.ZZ3 = ZZ3;

plotField.VV1 = VV1;
plotField.VV2 = VV2;
plotField.VV3 = VV3;



% PLOTTING!!!!!!!!
% This is quite tricky.. involves permuting matrices and stuff..

[XXX,YYY,ZZZ] = meshgrid(xvField,yvField,zvField);

VV1p = permute(VV1,[2,3,1]);
VVV1 = repmat(VV1p,1,numel(xvField),1);

VV2p = permute(VV2,[3,2,1]);
VVV2 = repmat(VV2p,numel(yvField),1,1);

VV3p = permute(VV3,[1,2,3]);
VVV3 = repmat(VV3p,1,1,numel(zvField));

figure
cont1 = contourslice(XXX,YYY,ZZZ,VVV1,XX1,YY1,ZZ1,50);
hold on
cont2 = contourslice(XXX,YYY,ZZZ,VVV2,XX2,YY2,ZZ2,50);
cont3 = contourslice(XXX,YYY,ZZZ,VVV3,XX3,YY3,ZZ3,50);

set(cont1, 'linewidth', 2) % Adjusting contourlines width
set(cont2, 'linewidth', 2)
set(cont3, 'linewidth', 2)

axis equal, axis off

MaxV = max([max(max(VV1)), max(max(VV2)), max(max(VV3))]);
MinV = min([min(min(VV1)), min(min(VV2)), min(min(VV3))]);
meanVal = 0.5*(MaxV+MinV);
for kk = 1:numel(sups) % Overimposing objects

   try
   
      surf(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, repmat(MaxV,size(sups(kk).xxc,1),size(sups(kk).xxc,2)));

   catch

      plot3(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, 'k', 'linewidth', 2)
   
   end
      
end

colorbar
xlabel('X','FontSize',14)
ylabel('Y','FontSize',14)
zlabel('Z','FontSize',14)
title('Electric Potential', 'FontSize',15)


fprintf('Done!\n')

