%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%






function [XYZq, XYZc, XXc, YYc, ZZc, Lmat, NN] = ring_creator(R,x,y,z,versor,n_th)

% ==== Wire Ring making ====
% 
%
% Step 1: creating a wire loop
% Step 2: rotating the wire
% Step 3: creating and rotating normals
% Step 4: creating Areas


%R=1.5,L=6,x=1,y=1,z-3,versor=[0,2,2],n_th=20,n_z=25

thvect0 = linspace(0,2*pi,n_th+1)';
thvect = thvect0(1:end-1);

deltaTh = 2*pi/n_th;

xq = R*cos(thvect);
yq = R*sin(thvect);
zq = zeros(size(xq));

xc = R*cos(thvect + 0.2*deltaTh);
yc = R*sin(thvect + 0.2*deltaTh);
zc = zeros(size(xq));

nnx = cos(thvect + 0.2*deltaTh);
nny = sin(thvect + 0.2*deltaTh);
nnz = zeros(size(thvect));

% Rotating

XYZq_O = [xq, yq, zq];
XYZc_O = [xc, yc, zc];

NN_O   = [nnx, nny, nnz];

XXc_O = xc;
YYc_O = yc;
ZZc_O = zc;



versor = versor/norm(versor); % Now it's a versor.
rotVers = cross([0,0,1],versor);
PSI1 = rotVers(1);
PSI2 = rotVers(2);
PSI3 = rotVers(3);


% Euler-Rodriguez formula
kcross = [0, -PSI3, PSI2; PSI3, 0, -PSI1; -PSI2, PSI1, 0];
thetaa = asin(norm(rotVers));

RRR = eye(3) + sin(thetaa)*kcross + (1-cos(thetaa))*kcross*kcross;
RotatedXYZq = (RRR*XYZq_O')';
RotatedXYZc = (RRR*XYZc_O')';
RotatedNN   = (RRR*NN_O')';

XYZq = RotatedXYZq;
XYZc = RotatedXYZc;
NN   = RotatedNN;

% Closing the wire loop for plotting reasons


XXc = [XYZc(:,1); XYZc(1,1)]';
YYc = [XYZc(:,2); XYZc(1,2)]';
ZZc = [XYZc(:,3); XYZc(1,3)]';


% Traslating everything
XYZq = XYZq + repmat([x,y,z], size(XYZq,1),1);
XYZc = XYZc + repmat([x,y,z], size(XYZq,1),1);

XXc = XXc + x;
YYc = YYc + y;
ZZc = ZZc + z;



% Length Vector
deltaL = R*acos(deltaTh);
Lmat   = repmat(deltaL, size(XXc,1), 1);


return
