%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% solution calculator


% MESSAGE

if verbosity == 1
   fprintf('\n====== Solution Computation ======\n');

   fprintf('Assembling problem matrices..');
end

wtAssembling = waitbar(0,'Assembling problem matrices..');

% Initializing matrices
AA = zeros(totalSups.nele, totalSups.nele);
CC = zeros(totalSups.nele, totalSups.nSupsFloating);
DD = zeros(totalSups.nSupsFloating, totalSups.nele);

bb = zeros(totalSups.nele + totalSups.nSupsFloating, 1);


% Assembling AA and CC matrices

posNow = 0;
posFloating = 1;

for kk = 1:(numel(sups)) % For every single surface


   % extracting kk control points
   rcNow = sups(kk).rc;


   for ii = 1:sups(kk).nele % for every control point
                               % of the surface

      rnow         = repmat(rcNow(ii,:), totalSups.nele, 1);
      DeltaRMATnow = totalSups.rq - rnow;
      DeltaRabs    = sqrt(DeltaRMATnow(:,1).^2 + DeltaRMATnow(:,2).^2 + DeltaRMATnow(:,3).^2);
         

      posNow	  = posNow + 1;
      AA(posNow,:) = 1/(4*pi*eps0) * 1./DeltaRabs;

   % Filling bb
      if isnumeric(sups(kk).V) % if Voltage is assigned

         bb(posNow)   = sups(kk).V;

      elseif strcmp(sups(kk).V,'free') % if Voltage set to 'free'
   
         bb(posNow)   = 0;
	 CC(posNow,posFloating) = - 1;
      
      end



   end
   
   if strcmp(sups(kk).V,'free')
      posFloating = posFloating + 1;
   end

      
   
   waitbar(posNow/(totalSups.nele+totalSups.nSupsFloating), wtAssembling);

end



% Assembling DD and the last part of bb

for kk = 1:numel(sups)

   if strcmp(sups(kk).V,'free')

      posNow   = posNow + 1;
      posNowDD = posNow - totalSups.nele;
    
      bb(posNow) = sups(kk).Q;
      
      DD(posNowDD,sups(kk).idstart:sups(kk).idend) = 1;
    
   end

end

if verbosity == 1
   fprintf('Done!\n');
end

close(wtAssembling)




% ================ SOLVING =================

if verbosity == 1
   fprintf('Solving... ');
end

MMat = [AA,CC; DD,zeros(totalSups.nSupsFloating)];
qVtot = MMat\bb;

qtot = qVtot(1:totalSups.nele);
Vtot = qVtot(totalSups.nele+1:end);

if verbosity == 1
   fprintf('Done!\n');
end

% Assigning q values to surfaces
for jj = 1:numel(sups)

   sups(jj).q = qtot(sups(jj).idstart : sups(jj).idend);

end

% Assigning V values to surfaces requesting it
poscount = 1;
for jj = 1:numel(sups)

   if strcmp(sups(jj).V, 'free')
   
      sups(jj).Vresult = Vtot(poscount);
      poscount = poscount + 1;

   else
   
      sups(jj).Vresult = sups(jj).V;
   
   end
   
end

totalSups.q = qtot;
totalSups.V = Vtot;




% ============== MESSAGES ================

if verbosity == 1
   fprintf('Solution was computed!\n')
end


for jj = 1:numel(sups)

   if isnumeric(sups(jj).V)
      Vnow = sups(jj).V;
   else
      Vnow = sups(jj).Vresult;
   end


   if verbosity == 1
      fprintf('Surface %d has:\n', jj)
      fprintf('  total charge [epsilon0 * q]: %f\n',sum(sups(jj).q))
      fprintf('  electric potential [V]: %f\n',Vnow)
   end


end
