%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





% sol_plotter


% MESSAGE
fprintf('\n====== Plotting Results ======\n')



% Step 0: creating reshaped solutions

fprintf('Rearranging solution matrices.. ');

clear jj % or I'm gonna make some mistake..

for kk = 1:numel(sups)

   qNow = sups(kk).q;
   xxNow = sups(kk).xxc;
   yyNow = sups(kk).yyc;
   zzNow = sups(kk).zzc;

   % reshaping the solution vector for plotting purposes
   
   if strcmp(sups(kk).type, 'sphere')
      
      sups(kk).qq = sphere_q_reshaper(qNow,xxNow,yyNow,zzNow);

   elseif strcmp(sups(kk).type, 'plane')
   
      sups(kk).qq = plane_q_reshaper(qNow,xxNow,yyNow,zzNow);
      
   elseif strcmp(sups(kk).type, 'cylinder')
   
      sups(kk).qq = cylinder_q_reshaper(qNow,xxNow,yyNow,zzNow); 

   elseif strcmp(sups(kk).type, 'cone')
   
      sups(kk).qq = cone_q_reshaper(qNow,xxNow,yyNow,zzNow);    
      
   end

end

% freeding a little memory
clear qNow xxNow yyNow zzNow kk

fprintf('Done!\n')




% =====  Complete plot  =====

fprintf('Plotting complete solution.. ');

cmpl_sol_plot = figure();
hold on

for kk = 1:numel(sups)

   % is it a surface? I can know it by looking at the
   % xxc dimensions!   
   minSize = min(size(sups(kk).xxc));
   
   if minSize > 1 % If it's a surface

      surf(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, sups(kk).qq ./ sups(kk).Amat);

   % Not a surface? Then plot it as a line!
   elseif minSize == 1

    plot3(sups(kk).xxc,sups(kk).yyc,sups(kk).zzc,'k','linewidth',2)

   end

end

grid on
xlabel('X', 'FontSize', 14)
ylabel('Y', 'FontSize', 14)
zlabel('Z', 'FontSize', 14)
title('Charge density - Epsilon_0 * q / Area','FontSize',15)
colorbar
axis equal

fprintf('Done!\n');


% ===== Partial Plot =====

for kk = 1:numel(sups)


   try

      if sups(kk).plot_me_alone == 1
      
         fprintf('Plotting solution for object %d.. ',kk);

         figure

%	 try
	 
	 % is it a surface? I can know it by looking at the
         % xxc dimensions!   
         minSize = min(size(sups(kk).xxc));
   
         if minSize > 1 % If it's a surface

            surf(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, sups(kk).qq ./ sups(kk).Amat);
            fprintf('Done!\n');

%	 catch unuseful_error
         elseif minSize == 1
	    plot3(sups(kk).xxc, sups(kk).yyc, sups(kk).zzc, 'k', 'linewidth', 2)
	    fprintf('You have tried to plot surface charge distribution for a line.. This program does not do that.\n');
%	    continue;
	 end
	 
         grid on
         xlabel('X', 'FontSize', 14)
         ylabel('Y', 'FontSize', 14)
         zlabel('Z', 'FontSize', 14)
         title(['Charge density for object ', num2str(kk), '- Epsilon_0 * q / Area'],'FontSize',15)
         colorbar
         axis equal
	 	 
      end	 
   catch unuseful_error
      continue
   end

end



% ===== Field Plotting =====


if plotField.plotPotential == 1

   potential_plotter

end


if plotField.plotElectricField == 1

   electfield_plotter

end   
