%
% Copyright (C) 2015 Stefano Boccelli <dainonottambulo -at- gmail.com>
% 
% This program is part of Qsci, which is free software: 
% you can redistribute it and/or modify it under the terms of 
% the GNU Lesser General Public License as published by the 
% Free Software Foundation (version 3 of the License).
%
% Qsci is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  
% If not, see <http://www.gnu.org/licenses/>.
%





function [XYZq, XYZc, XXc, YYc, ZZc, Amat, NN] = sphere_creator(R,x,y,z,n_th,n_phi)

% ==== Sphere making ====



thvect0 = linspace(0,pi,n_th+2)';
thvect = thvect0(2:end-1);
%thvect = thvect(1:n_th); % NON SCOMMENTARE, E' GIUSTO COSI'

phivect0 = linspace(0,2*pi,n_phi+1)';
phivect = phivect0(1:n_phi);

d_ar = 0.1 * 2*pi*R/n_th;
%d_ar = sin(thvect) * 0.5*2*pi*R/n_th; % Distanza proporzionale alla
                                      % quindi varia con theta.                                      
                                      
phiMAT = repmat(phivect,1,n_th);
thMAT  = repmat(thvect',n_phi,1);


rthphiMAT = [R*ones(1,numel(phiMAT)), phiMAT(1:end), thMAT(1:end)];

xxq = x + R*sin(thMAT).*cos(phiMAT);
yyq = y + R*sin(thMAT).*sin(phiMAT);
zzq = z + R*cos(thMAT);

xxc = x + (R+d_ar)*sin(thMAT).*cos(phiMAT);
yyc = y + (R+d_ar)*sin(thMAT).*sin(phiMAT);
zzc = z + (R+d_ar)*cos(thMAT);

XYZq = [xxq(1:end)', yyq(1:end)', zzq(1:end)'];
XYZc = [xxc(1:end)', yyc(1:end)', zzc(1:end)'];


% Closing the sphere for plotting vectors
XXc = [xxc; xxc(1,:)];
YYc = [yyc; yyc(1,:)];
ZZc = [zzc; zzc(1,:)];


% Adding poles to charge and control points vectors
XYZq = [ [x, y, z + R]; XYZq; [x, y, z - R]];
XYZc = [ [x, y, z + R+d_ar]; XYZc; [x, y, z - (R+d_ar)]];




% ---- Computing Normals ----

Nxx = sin(thMAT).*cos(phiMAT);
Nyy = sin(thMAT).*sin(phiMAT);
Nzz = cos(thMAT);

NN = [Nxx(1:end)',Nyy(1:end)',Nzz(1:end)'];
NN = [[0,0,1];NN;[0,0,-1]]; % Adding poles

% ---- Computing area ----

LatMat  = 180/pi*thMAT;
LongMat = 180/pi*phiMAT;

Amat = zeros(numel(thvect0)-2,numel(phivect0));
for ii = 1:numel(thvect0)-2
   th0 = 180/pi*thvect0(ii+1);
   th1 = 180/pi*thvect0(ii+2);
   
   lat0 = 90 - th0;
   lat1 = 90 - th1;
   for jj = 1:numel(phivect0)-1
       phi0 = 180/pi*phivect0(jj);
       phi1 = 180/pi*phivect0(jj+1);
       long0 = phi0;
       long1 = phi1;
       
       Aiijj = R*areaquad(lat0,long0,lat1,long1);
       Amat(ii,jj) = Aiijj;        
   end
end

% Filling last line
Amat(end,:) = Amat(end-1,:);
% ... and last column
Amat(:,end) = Amat(:,end-1);

Amat = Amat';

for ii = 1:numel(Amat)
    if Amat(ii) < 1E-10
        Amat(ii) = NaN;
    end
end






return
